@@ -1,17 +1,18 @@
-from zope import schema
-from zope.formlib import form
-from zope.interface import implements, Interface
-
-from plone.app.portlets.portlets import base
-from plone.portlets.interfaces import IPortletDataProvider
-
-from Products.CMFPlone import PloneMessageFactory as _
-from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
-
-import feedparser
 import time
 
+import feedparser
+from plone.portlets.interfaces import IPortletDataProvider
+from zope.formlib import form
+from zope.interface import implements, Interface
+from zope import schema
+
 from DateTime import DateTime
+from DateTime.interfaces import DateTimeError
+from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
+
+from plone.app.portlets import PloneMessageFactory as _
+from plone.app.portlets.portlets import base
+
 
 # Accept these bozo_exceptions encountered by feedparser when parsing
 # the feed:
@@ -20,6 +21,7 @@ ACCEPTED_FEEDPARSER_EXCEPTIONS = (feedparser.CharacterEncodingOverride, )
 # store the feeds here (which means in RAM)
 FEED_DATA = {}  # url: ({date, title, url, itemlist})
 
+
 class IFeed(Interface):
 
     def __init__(url, timeout):
@@ -62,6 +64,7 @@ class IFeed(Interface):
     def ok():
         """is this feed ok to display?"""
 
+
 class RSSFeed(object):
     """an RSS feed"""
     implements(IFeed)
@@ -127,6 +130,24 @@ class RSSFeed(object):
 
         return self.ok
 
+    def _buildItemDict(self, item):
+        link = item.links[0]['href']
+        itemdict = {
+            'title': item.title,
+            'url': link,
+            'summary': item.get('description', ''),
+        }
+        if hasattr(item, "updated"):
+            try:
+                itemdict['updated'] = DateTime(item.updated)
+            except DateTimeError:
+                # It's okay to drop it because in the
+                # template, this is checked with
+                # ``exists:``
+                pass
+
+        return itemdict
+
     def _retrieveFeed(self):
         """do the actual work and try to retrieve the feed"""
         url = self.url
@@ -134,7 +155,7 @@ class RSSFeed(object):
             self._last_update_time_in_minutes = time.time()/60
             self._last_update_time = DateTime()
             d = feedparser.parse(url)
-            if d.bozo == 1 and not isinstance(d.get('bozo_exception'),
+            if getattr(d, 'bozo', 0) == 1 and not isinstance(d.get('bozo_exception'),
                                               ACCEPTED_FEEDPARSER_EXCEPTIONS):
                 self._loaded = True # we tried at least but have a failed load
                 self._failed = True
@@ -144,16 +165,10 @@ class RSSFeed(object):
             self._items = []
             for item in d['items']:
                 try:
-                    link = item.links[0]['href']
-                    itemdict = {
-                        'title' : item.title,
-                        'url' : link,
-                        'summary' : item.get('description',''),
-                    }
-                    if hasattr(item, "updated"):
-                        itemdict['updated']=DateTime(item.updated)
+                    itemdict = self._buildItemDict(item)
                 except AttributeError:
                     continue
+
                 self._items.append(itemdict)
             self._loaded = True
             self._failed = False
@@ -162,7 +177,6 @@ class RSSFeed(object):
         self._failed = True # no url set means failed
         return False # no url set, although that actually should not really happen
 
-
     @property
     def items(self):
         return self._items
@@ -173,7 +187,7 @@ class RSSFeed(object):
     @property
     def feed_link(self):
         """return rss url of feed for portlet"""
-        return self.url.replace("http://","feed://")
+        return self.url.replace("http://", "feed://")
 
     @property
     def title(self):
@@ -185,14 +199,14 @@ class RSSFeed(object):
         """return the link to the site the RSS feed points to"""
         return self._siteurl
 
+
 class IRSSPortlet(IPortletDataProvider):
 
     portlet_title = schema.TextLine(
         title=_(u'Title'),
         description=_(u'Title of the portlet.  If omitted, the title of the feed will be used.'),
         required=False,
-        default=u''
-        )
+        default=u'')
 
     count = schema.Int(title=_(u'Number of items to display'),
                        description=_(u'How many items to list.'),
@@ -208,6 +222,7 @@ class IRSSPortlet(IPortletDataProvider):
                         required=True,
                         default=100)
 
+
 class Assignment(base.Assignment):
     implements(IRSSPortlet)
 
@@ -216,7 +231,7 @@ class Assignment(base.Assignment):
     @property
     def title(self):
         """return the title with RSS feed title or from URL"""
-        feed = FEED_DATA.get(self.data.url,None)
+        feed = FEED_DATA.get(self.data.url, None)
         if feed is None:
             return u'RSS: '+self.url[:20]
         else:
@@ -228,6 +243,7 @@ class Assignment(base.Assignment):
         self.url = url
         self.timeout = timeout
 
+
 class Renderer(base.DeferredRenderer):
 
     render_full = ZopeTwoPageTemplateFile('rss.pt')
@@ -254,10 +270,10 @@ class Renderer(base.DeferredRenderer):
 
     def _getFeed(self):
         """return a feed object but do not update it"""
-        feed = FEED_DATA.get(self.data.url,None)
+        feed = FEED_DATA.get(self.data.url, None)
         if feed is None:
             # create it
-            feed = FEED_DATA[self.data.url] = RSSFeed(self.data.url,self.data.timeout)
+            feed = FEED_DATA[self.data.url] = RSSFeed(self.data.url, self.data.timeout)
         return feed
 
     @property
@@ -265,7 +281,6 @@ class Renderer(base.DeferredRenderer):
         """return url of feed for portlet"""
         return self._getFeed().url
 
-
     @property
     def siteurl(self):
         """return url of site for portlet"""
@@ -274,7 +289,7 @@ class Renderer(base.DeferredRenderer):
     @property
     def feedlink(self):
         """return rss url of feed for portlet"""
-        return self.data.url.replace("http://","feed://")
+        return self.data.url.replace("http://", "feed://")
 
     @property
     def title(self):
@@ -294,17 +309,18 @@ class Renderer(base.DeferredRenderer):
     def enabled(self):
         return self._getFeed().ok
 
+
 class AddForm(base.AddForm):
     form_fields = form.Fields(IRSSPortlet)
     label = _(u"Add RSS Portlet")
     description = _(u"This portlet displays an RSS feed.")
 
-
     def create(self, data):
         return Assignment(portlet_title=data.get('portlet_title', u''),
                           count=data.get('count', 5),
-                          url = data.get('url',''),
-                          timeout = data.get('timeout',100))
+                          url = data.get('url', ''),
+                          timeout = data.get('timeout', 100))
+
 
 class EditForm(base.EditForm):
     form_fields = form.Fields(IRSSPortlet)
